/* Project name:
     Calculator plus Led_Blinking 2.0 (The even more complex simple example)
 * Copyright:
     (c) 
 * Modified by:
      Stevan A. Milinkovic
      Union University, School of Computing
      Knez Mihailova 6, 11000 Belgrade, Serbia
      e-mail: smilinkovic@raf.edu.rs
 * Revision History:
       - initial release;
 * Description:
     Multitasking example for the world of ARM MCUs using FreeRTOS;
 * Test configuration:
     Device:          STM32F107VC
                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
     Dev.Board:       EasyMx v7 for STM(R) ARM(R)
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HSE-PLL, 72.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/eng/products/view/752/mikroc-pro-for-arm/
 * NOTES:
     - Turn ON PORTA, PORTB, PORTC, PORTD, PORTE at SW15 (board specific).
     - Turn on SW11.3 (display back light) to SW11.8 (board specific)
 */


#include "FreeRTOS.h"
#include "task.h"
#include "calculator_objects.h"

#define ledSTACK_SIZE               configMINIMAL_STACK_SIZE
#define mainBLINKING_TASK_PRIORITY  ( tskIDLE_PRIORITY + 2 )
#define CALC_TASK_PRIORITY          ( tskIDLE_PRIORITY + 1 )
#define TaskDelay_ms(x)             vTaskDelay( x/portTICK_RATE_MS )


static portTASK_FUNCTION( vCalculatorTask, pvParameters ) {
  for (;;) {
    Check_TP();
  }
}

static portTASK_FUNCTION( vLedBlinkingTask, pvParameters ) {
  for (;;) {
    GPIOC_ODR = ~GPIOD_ODR; // Toggle PORTC
    GPIOD_ODR = ~GPIOD_ODR; // Toggle PORTD
    TaskDelay_ms(1000);
  }
}


void main(void ) {
  Start_TP();

  GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_ALL); // Set PORTC as digital output
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_ALL); // Set PORTD as digital output
  GPIOC_ODR = 0;
  GPIOD_ODR = 0;
  
  xTaskCreate( vCalculatorTask,  ( signed char * ) "CALC", ledSTACK_SIZE, NULL, CALC_TASK_PRIORITY, ( xTaskHandle * ) NULL );
  xTaskCreate( vLedBlinkingTask, ( signed char * ) "LED",  ledSTACK_SIZE, NULL, mainBLINKING_TASK_PRIORITY, ( xTaskHandle * ) NULL );
  vTaskStartScheduler();
}